"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Babel = require("babel-core");
const fs_1 = require("fs");
const glob_1 = require("glob");
const path_1 = require("path");
const resolve_1 = require("resolve");
exports.DEFAULT_EXTENSIONS = new Set(['.js', '.jsx']);
class Plugin {
    constructor(rawPlugin, inferredName, path) {
        this.rawPlugin = rawPlugin;
        this.inferredName = inferredName;
        this.path = path;
        let instance = rawPlugin(Babel);
        if (instance.name) {
            this.declaredName = instance.name;
        }
    }
    static load(path, inferredName) {
        let exports = require(path);
        let plugin;
        if (exports.default) {
            plugin = exports.default;
        }
        else {
            plugin = exports;
        }
        let rawPlugin = plugin;
        return new Plugin(rawPlugin, inferredName, path);
    }
}
exports.Plugin = Plugin;
class DeferredPlugin {
    constructor(path, inferredName) {
        this.path = path;
        this.inferredName = inferredName;
    }
    load() {
        if (!this.loaded) {
            this.loaded = Plugin.load(this.path, this.inferredName);
        }
        return this.loaded;
    }
}
exports.DeferredPlugin = DeferredPlugin;
class Options {
    constructor(sourcePaths, plugins, pluginOptions, extensions, requires, ignore, stdio, help) {
        this.sourcePaths = sourcePaths;
        this.plugins = plugins;
        this.pluginOptions = pluginOptions;
        this.extensions = extensions;
        this.requires = requires;
        this.ignore = ignore;
        this.stdio = stdio;
        this.help = help;
    }
    getPlugins() {
        return this.plugins.map(plugin => plugin.load());
    }
    loadRequires() {
        for (let modulePath of this.requires) {
            require(modulePath);
        }
    }
    getPlugin(name) {
        for (let plugin of this.getPlugins()) {
            if (plugin.declaredName === name || plugin.inferredName === name) {
                return plugin;
            }
        }
        return null;
    }
    getBabelPlugins() {
        let result = [];
        for (let plugin of this.getPlugins()) {
            let options = plugin.declaredName &&
                this.pluginOptions.get(plugin.declaredName) ||
                this.pluginOptions.get(plugin.inferredName);
            if (options) {
                result.push([plugin.rawPlugin, options]);
            }
            else {
                result.push(plugin.rawPlugin);
            }
        }
        return result;
    }
    getBabelPlugin(name) {
        let plugin = this.getPlugin(name);
        if (!plugin) {
            return null;
        }
        let options = this.pluginOptions.get(name);
        if (options) {
            return [plugin.rawPlugin, options];
        }
        else {
            return plugin.rawPlugin;
        }
    }
    static parse(args) {
        let sourcePaths = [];
        let plugins = [];
        let pluginOptions = new Map();
        let extensions = exports.DEFAULT_EXTENSIONS;
        let ignore = (path, basename, root) => basename[0] === '.';
        let requires = [];
        let stdio = false;
        let help = false;
        for (let i = 0; i < args.length; i++) {
            let arg = args[i];
            switch (arg) {
                case '-p':
                case '--plugin':
                    i++;
                    let path = args[i];
                    plugins.push(new DeferredPlugin(getRequirableModulePath(path), path_1.basename(path, path_1.extname(path))));
                    break;
                case '-o':
                case '--plugin-options':
                    i++;
                    let nameAndOptions = args[i].split('=');
                    let name = nameAndOptions[0];
                    let optionsRaw = nameAndOptions[1];
                    if (optionsRaw && optionsRaw[0] === '@') {
                        optionsRaw = fs_1.readFileSync(optionsRaw.slice(1), { encoding: 'utf8' });
                        pluginOptions.set(name, JSON.parse(optionsRaw));
                    }
                    try {
                        pluginOptions.set(name, JSON.parse(optionsRaw));
                    }
                    catch (err) {
                        return new Error(`unable to parse JSON config for ${name}: ${optionsRaw}`);
                    }
                    break;
                case '-r':
                case '--require':
                    i++;
                    requires.push(getRequirableModulePath(args[i]));
                    break;
                case '--extensions':
                    i++;
                    extensions = new Set(args[i]
                        .split(',')
                        .map(ext => ext[0] === '.' ? ext : `.${ext}`));
                    break;
                case '-s':
                case '--stdio':
                    stdio = true;
                    break;
                case '-h':
                case '--help':
                    help = true;
                    break;
                default:
                    if (arg[0] === '-') {
                        return new Error(`unexpected option: ${arg}`);
                    }
                    else {
                        if (glob_1.hasMagic(arg)) {
                            sourcePaths.push(...glob_1.sync(arg));
                        }
                        else {
                            sourcePaths.push(arg);
                        }
                    }
                    break;
            }
        }
        return new Options(sourcePaths, plugins, pluginOptions, extensions, requires, ignore, stdio, help);
    }
}
exports.default = Options;
/**
 * Gets a path that can be passed to `require` for a given module path.
 */
function getRequirableModulePath(modulePath) {
    if (fs_1.existsSync(modulePath) || fs_1.existsSync(modulePath + '.js')) {
        return path_1.resolve(modulePath);
    }
    else {
        return resolve_1.sync(modulePath, { basedir: process.cwd() });
    }
}
//# sourceMappingURL=Options.js.map