"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path_1 = require("path");
const iterateSources_1 = require("./iterateSources");
const Options_1 = require("./Options");
const TransformRunner_1 = require("./TransformRunner");
function printHelp(out) {
    let $0 = path_1.basename(process.argv[1]);
    out.write(`
${$0} [OPTIONS] [PATH … | --stdio]

OPTIONS
  -p, --plugin PLUGIN               Transform sources with PLUGIN (allows multiple).
  -o, --plugin-options PLUGIN=OPTS  JSON-encoded OPTS for PLUGIN (allows multiple).
  -r, --require PATH                Require PATH before transform (allows multiple).
      --extensions EXTS             Comma-separated extensions to process (default: "${Array.from(Options_1.DEFAULT_EXTENSIONS).join(',')}").
  -s, --stdio                       Read source from stdin and print to stdout.
  -h, --help                        Show this help message.

EXAMPLES
  # Run with a relative plugin on all files in \`src/\`.
  $ ${$0} -p ./typecheck.js src/

  # Run with multiple plugins.
  $ ${$0} -p ./a.js -p ./b.js some-file.js

  # Run with a plugin in \`node_modules\` on stdin.
  $ ${$0} -s -p babel-plugin-typecheck <<EOS
  function add(a: number, b: number): number {
    return a + b;
  }
  EOS

  # Pass options to a plugin.
  $ ${$0} -p ./a.js -o a='{"foo":true}' src/

  # Pass options from a config file to a plugin.
  $ ${$0} -p ./a.js -o a=@opts.json src/

  # Run with a plugin which itself is transpiled using babel.
  $ ${$0} -r babel-register -p ./some-plugin.js src/

  # Run with a plugin written in TypeScript.
  $ ${$0} -r ts-node/register -p ./some-plugin.ts src/
  `.trim());
    out.write('\n');
}
function run(args) {
    return __awaiter(this, void 0, void 0, function* () {
        let options = Options_1.default.parse(args);
        if (options instanceof Error) {
            process.stderr.write(`ERROR: ${options.message}\n`);
            printHelp(process.stderr);
            process.exit(1);
            return;
        }
        if (options.help) {
            printHelp(process.stdout);
            process.exit(0);
            return;
        }
        options.loadRequires();
        let plugins = options.getBabelPlugins();
        let runner;
        if (options.stdio) {
            runner = new TransformRunner_1.default([new TransformRunner_1.Source('<stdin>', yield readStdin())][Symbol.iterator](), plugins, {
                transformSourceEnd(runner, transformed) {
                    if (transformed.output) {
                        process.stdout.write(transformed.output);
                    }
                    else if (transformed.error) {
                        console.error(transformed.error.stack);
                    }
                }
            });
        }
        else {
            runner = new TransformRunner_1.default(iterateSources_1.default(options.sourcePaths, options.extensions, options.ignore), plugins, {
                transformSourceEnd(runner, transformed) {
                    if (transformed.output) {
                        if (transformed.output !== transformed.source.content) {
                            console.log(transformed.source.path);
                            fs_1.writeFileSync(transformed.source.path, transformed.output);
                        }
                    }
                    else if (transformed.error) {
                        console.error(`Encountered an error while processing ${transformed.source.path}:`);
                        console.error(transformed.error.stack);
                    }
                }
            });
        }
        let hasErrors = false;
        for (let result of runner.run()) {
            if (result.error !== null) {
                hasErrors = true;
            }
        }
        process.exit(hasErrors ? 1 : 0);
    });
}
exports.default = run;
/**
 * Reads stdin and resolves to the read string.
 */
function readStdin() {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise(resolve => {
            let code = '';
            process.stdin.on('data', data => {
                code += data;
            });
            process.stdin.on('end', () => {
                resolve(code);
            });
        });
    });
}
//# sourceMappingURL=index.js.map